searchData={"items":[{"type":"behaviour","title":"Kiq","doc":"Kiq is a robust and extensible job processing queue that aims for\ncompatibility with Sidekiq Enterprise.\n\nJob queuing, processing and reporting are all built on GenStage. That means\nmaximum parallelism with the safety of backpressure as jobs are processed.","ref":"Kiq.html"},{"type":"behaviour","title":"Usage - Kiq","doc":"Kiq isn't an application that must be started. Similarly to Ecto, you define\none or more Kiq modules within your application. This allows multiple\nsupervision trees with entirely different configurations.\n\nRun the generator to create a new `Kiq` supervisor for your application:\n\n    mix kiq.gen.supervisor -m MyApp.Kiq\n\nAlternatively, manually define a `Kiq` module for your application:\n\n    defmodule MyApp.Kiq do\n      use Kiq, queues: [default: 25, events: 50]\n    end\n\nInclude the module in your application's supervision tree:\n\n    defmodule MyApp.Application do\n      @moduledoc false\n\n      use Application\n\n      alias MyApp.{Endpoint, Kiq, Repo}\n\n      def start(_type, _args) do\n        children = [\n          {Repo, []},\n          {Endpoint, []},\n          {Kiq, []}\n        ]\n\n        Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n      end\n    end","ref":"Kiq.html#module-usage"},{"type":"behaviour","title":"Configuration - Kiq","doc":"Kiq is used to start one or more supervision trees in your application. That\nmeans there isn't a central Kiq \"app\" to configure, instead each supervision\ntree may be configured independently.\n\nConfiguration options pass through a couple of locations, accumulating\noverrides until finally passing through the `init/2` callback. Options are\naccumulated in this order:\n\n1. Options passed into via the `use` macro. These should be constant compile\n   time options, i.e. `extra_reporters`.\n\n2. Options passed to `start_link/1` by the application's supervision tree.\n   These should also be values suitable for compile time.\n\n3. Injected by the `init/2` callback inside your application's Kiq instance.\n   This is where runtime configuration such as the Redis URL or environment\n   specific options should be passed. The options can come from any dynamic\n   source such as Mix Config, Vault, Etcd, environment variables, etc.\n\nThe default `init/2` implementation uses `System.get_env/1` to read the\n`REDIS_URL` on boot. The default callback can be overridden to pull in\nadditional configuration. For example, to grab values from\n`Application.get_env/2`:\n\n    def init(_reason, opts) do\n      for_env = Application.get_env(:my_app, :kiq, [])\n\n      opts =\n        opts\n        |> Keyword.merge(for_env)\n        |> Keyword.put(:client_opts, [redis_url: System.get_env(\"REDIS_URL\")])\n\n      {:ok, opts}\n    end\n\nThe `opts` argument contains all configuration from stages 1 and 2 (the `use`\nmacro and the call to `start_link/1`).\n\n#","ref":"Kiq.html#module-configuration"},{"type":"behaviour","title":"Supervisor Configuration - Kiq","doc":"These configuration options must be provided to the supervision tree on startup:\n\n* `:client_opts` — A keyword list of options passed to each `Redix`\n  connection.  This *must* contain the key `:redis_url`, which is passed to\n  `Redix.start_link/1` as the first argument.\n\n* `:dead_limit` — The maximum number of jobs that will be retained in the dead\n  set. Jobs beyond the limit are pruned any time a new job is moved to the dead\n  set. The default is `10_000`.\n\n* `:dead_timeout` — The maximum amount of time that a job will remain in the\n  dead set before being purged, specified in seconds. The default is 6 months.\n\n* `:extra_reporters` — Additional reporters that your application will use to\n  report errors, track external stats, etc.\n\n* `:fetch_interval` — How frequently to poll for new jobs. Polling only\n  happens when consumers aren't actively requesting new jobs.\n\n* `:flush_interval` - How frequently locally enqueued jobs will be pushed to\n  Redis. This defaults to `10ms`, though it will back-off by a factor of `1.5`\n  if there are any connection errors.\n\n* `:periodics` — A list of job scheduling tuples in the form `{schedule,\n  worker}` or `{schedule, worker, options}`. See\n  [Periodic Jobs](#module-periodic-jobs) for details.\n\n* `:pool_size` — Controls the number of Redis connections available to Kiq,\n  defaults to 5.\n\n* `:queues` — A keyword list of queues where each entry is the name of the\n  queue and the concurrency setting. For example, setting `[default: 10,\n  exports: 5, media: 5]` would start the queues `default`, `exports` and\n  `media` with a combined concurrency of 20. The concurrency setting\n  specifies how many jobs _each queue_ will run concurrently.\n\n* `:schedulers` — A list of schedulers to run. The default schedulers are\n  \"retry\" which is used to retry failed jobs with a backoff and \"schedule\",\n  used to enqueue jobs at a specific time in the future. Set this to an empty\n  list to disable all schedulers and allow Sidekiq to handle enqueuing\n  retries and scheduled jobs.\n\n* `:server?` — Whether to start the queue supervisors and start processing\n  jobs or only start the client. This setting is useful for testing or\n  deploying your application's web and workers separately.\n\n* `:test_mode` — Either `:disabled` or `:sandbox`. See\n  [Testing](#module-testing) for details.\n\n#","ref":"Kiq.html#module-supervisor-configuration"},{"type":"behaviour","title":"Runtime Configuration - Kiq","doc":"These configuration options may be provided when starting a Kiq supervisor,\nbut may also be set dynamically at runtime:\n\n* `:quiet` — Instruct each queue to stop processing new jobs. Any in-progress\njobs will keep running uninterrupted.\n\nFor example, to quiet all queue producers:\n\n    MyKiq.configure(quiet: true)","ref":"Kiq.html#module-runtime-configuration"},{"type":"behaviour","title":"Testing - Kiq","doc":"Kiq has special considerations to facilitate isolated and asynchronous testing.\n\nFor testing Kiq should be configured in `:sandbox` mode and have the server\ndisabled. This can be done specifically for the test environment by adding\nconfig to `config/test.exs`:\n\n    config :my_app, :kiq, server?: false, test_mode: :sandbox\n\nRunning in `:sandbox` mode ensures that enqueued jobs stay in memory and are\nnever flushed to Redis. This allows your tests to use `Kiq.Testing` to make\nquick assertions about which jobs have or haven't been enqueued. See the docs\nfor `Kiq.Testing` for more details and usage.","ref":"Kiq.html#module-testing"},{"type":"behaviour","title":"Reliable Push - Kiq","doc":"Reliable push replicates the safety aspects of Sidekiq Pro's [reliability\nclient][rely]. To guard against network errors or other Redis issues the\nclient buffers all jobs locally. At a frequent interval the jobs are flushed\nto Redis. If there are any errors while flushing the jobs will remain in\nmemory until flushing can be retried later.\n\n#","ref":"Kiq.html#module-reliable-push"},{"type":"behaviour","title":"Caveats - Kiq","doc":"* The local job buffer is stored in memory, if the server is restarted\n    suddently some jobs may be lost.\n\n  * There isn't any limit on the number of jobs that can be buffered.\n    However, to conserve space jobs are stored compressed.\n\n[rely]: https://github.com/mperham/sidekiq/wiki/Pro-Reliability-Client","ref":"Kiq.html#module-caveats"},{"type":"behaviour","title":"Private Queues - Kiq","doc":"Kiq tries to prevent all job loss through private queues, a variant of the\n[Super Fetch][super] mechanism available in Sidekiq Pro. When jobs are\nexecuted they are backed up to a private queue specific to the server\nprocessing the job. If the processor crashes or the application is terminated\nbefore the job is finished the jobs remain backed up. On startup, and\nperiodically afterward, jobs in any dead private queues are pushed back to\nthe public queue for re-execution.\n\nThis solution is well suited to containerized environments and autoscaling.\n\n[super]: https://github.com/mperham/sidekiq/wiki/Pro-Reliability-Server#super_fetch","ref":"Kiq.html#module-private-queues"},{"type":"behaviour","title":"Unique Jobs - Kiq","doc":"Kiq supports Sidekiq Enterprise's [unique jobs][uniq]. This feature prevents\nenqueueing duplicate jobs while an original job is still pending. The\noperations that attempt to enforce uniqueness are _not_ atomic—uniquess is\nnot guaranteed and should be considered best effort.\n\nEnable unique jobs for a worker by setting a unique time period:\n\n    use Kiq.Worker, unique_for: :timer.minutes(5)\n\n#","ref":"Kiq.html#module-unique-jobs"},{"type":"behaviour","title":"Unlock Policy - Kiq","doc":"By default unique jobs will hold the unique lock until the job has ran\nsuccessfully. This policy ensures that jobs will remain locked if there are\nany errors, but it is prone to race conditions in certain situations for\nlonger running jobs.\n\nGenerally it is best to stick with the default `:success` policy. However, if\nyour job is effected by the race condition you can change your worker's\npolicy:\n\n    use Kiq.Worker, unique_for: :timer.minutes(60), unique_until: :start\n\n#","ref":"Kiq.html#module-unlock-policy"},{"type":"behaviour","title":"Caveats - Kiq","doc":"* Note that job uniqueness is calculated from the `class`, `args`, and\n    `queue`. This means that jobs with identical args may be added to different\n    queues.\n\n  * Unique jobs enqueued by Sidekiq will be unlocked by Kiq, but they may not\n    use the same lock value. This is due to differences between hashing Erlang\n    terms and Ruby objects. To help ensure uniqueness always enqueue unique jobs\n    from either Sidekiq or Kiq.\n\n[uniq]: https://github.com/mperham/sidekiq/wiki/Ent-Unique-Jobs","ref":"Kiq.html#module-caveats"},{"type":"behaviour","title":"Expiring Jobs - Kiq","doc":"Kiq supports Sidekiq Pro's [expiring jobs][expi]. Expiring jobs won't be ran\nafter a configurable amount of time. The expiration period is set with the\n`expires_in` option, which accepts millisecond values identically\n`unique_for`.\n\nSet the expiration for a worker using a relative time:\n\n    use Kiq.Worker, expires_in: :timer.hours(1)\n\nExpiration time applies after the scheduled time. That means scheduling a job\nto run in an hour, with an expiration of 30 minutes, will expire in one hour\nand 30 minutes.\n\n[expi]: https://github.com/mperham/sidekiq/wiki/Pro-Expiring-Jobs","ref":"Kiq.html#module-expiring-jobs"},{"type":"behaviour","title":"Periodic Jobs - Kiq","doc":"Kiq supports Sidekiq Enterprise's [Periodic Jobs][peri]. This allows jobs to\nbe registered with a schedule and enqueued automatically. Jobs are registered\nas `{crontab, worker}` or `{crontab, worker, options}` using the `:periodics`\nattribute:\n\n    use Kiq, periodics: [\n      {\"* * * * *\", MyApp.MinuteWorker},\n      {\"0 * * * *\", MyApp.HourlyWorker},\n      {\"0 0 * * *\", MyApp.DailyWorker, retry: 1},\n    ]\n\nThese jobs would be executed as follows:\n\n  * `MyApp.MinuteWorker` - Executed once every minute\n  * `MyApp.HourlyWorker` - Executed at the first minute of every hour\n  * `MyApp.DailyWorker` - Executed at midnight every day\n\nThe crontab format, as parsed by `Kiq.Parser.Crontab`, respects all [standard\nrules][cron] and has one minute resolution. That means it isn't possible to\nenqueue a job ever N seconds.\n\n#","ref":"Kiq.html#module-periodic-jobs"},{"type":"behaviour","title":"Caveats - Kiq","doc":"* All schedules are evaluated as UTC, the local timezone is never taken\n    into account.\n\n  * Periodic jobs registered in Kiq _aren't_ visible in the Loop panel of the\n    Sidekiq Dashboard. This is due to the way loop data is stored by Sidekiq\n    and can't be worked around.\n\n  * This is an alternative to using using a separate scheduler such as\n    [Quantum][quan]. However, unlike Quantum, Kiq doesn't support node based\n    clustering, instead it uses Redis to coordinate and distrubte work. This\n    means workers can scale horizontally even in a restricted environment like\n    Heroku.\n\n[peri]: https://github.com/mperham/sidekiq/wiki/Ent-Periodic-Jobs\n[cron]: https://en.wikipedia.org/wiki/Cron#Overview\n[quan]: https://github.com/quantum-elixir/quantum-core","ref":"Kiq.html#module-caveats"},{"type":"behaviour","title":"Instrumentation & Metrics - Kiq","doc":"The instrumentation reporter provides integration with [Telemetry][tele], a\ndispatching library for metrics. It is easy to report Kiq metrics to any\nbackend by attaching to `:kiq` events.\n\nFor exmaple, to log out the timing for all successful jobs:\n\n    defmodule KiqJobLogger do\n      require Logger\n\n      def handle_event([:kiq, :job, :success], timing, metadata, _config) do\n        Logger.info(\"[#{metadata.queue}] #{metadata.class} finished in #{timing}\")\n      end\n    end\n\n    :telemetry.attach([:kiq, :job, :success], &KiqJobLogger.handle_event/4, nil)\n\nHere is a reference for the available metrics:\n\n| event     | name                     | value  | metadata                  |\n| --------- | ------------------------ | ------ | ------------------------- |\n| `started` | `[:kiq, :job, :started]` | 1      | `:class, :queue`          |\n| `success` | `[:kiq, :job, :success]` | timing | `:class, :queue`          |\n| `aborted` | `[:kiq, :job, :aborted]` | 1      | `:class, :queue, :reason` |\n| `failure` | `[:kiq, :job, :failure]` | 1      | `:class, :queue, :error`  |\n\n[tele]: https://hexdocs.pm/telemetry","ref":"Kiq.html#module-instrumentation-metrics"},{"type":"callback","title":"Kiq.clear/0","doc":"Clear all enqueued, scheduled and backup jobs.\n\nAll known queues are cleared, even if they aren't listed in the current configuration.","ref":"Kiq.html#c:clear/0"},{"type":"callback","title":"Kiq.configure/1","doc":"Set runtime configuration values.\n\nSee the \"Runtime Configuration\" section in the `Kiq` module documentation.","ref":"Kiq.html#c:configure/1"},{"type":"callback","title":"Kiq.enqueue/2","doc":"Enqueue a job to be processed asynchronously.\n\nJobs can be enqueued from `Job` structs, maps or keyword lists.","ref":"Kiq.html#c:enqueue/2"},{"type":"callback","title":"Options - Kiq.enqueue/2","doc":"* `in` - The amount of time in seconds to wait before processing the job. This must be a\n  positive integer.\n* `at` - A specific `DateTime` in the future when the job should be processed.","ref":"Kiq.html#c:enqueue/2-options"},{"type":"callback","title":"Examples - Kiq.enqueue/2","doc":"# Enqueue a job to be processed immediately\n    MyJob.new([1, 2]) |> MyKiq.enqueue()\n\n    # Enqueue a job in one minute\n    MyJob.new([1, 2]) |> MyKiq.enqueue(in: 60)\n\n    # Enqueue a job some time in the future\n    MyJob.new([1, 2]) |> MyKiq.enqueue(at: ~D[2020-09-20 12:00:00])\n\n    # Enqueue a job from scratch, without using a worker module\n    MyKiq.enqueue(class: \"ExternalWorker\", args: [1])","ref":"Kiq.html#c:enqueue/2-examples"},{"type":"callback","title":"Kiq.init/2","doc":"A callback executed when the supervision tree is started and possibly when configuration is\nread.\n\nThe first argument is the context of the callback being invoked. In most circumstances this\nwill be `:supervisor`. The second argument is a keyword list of the combined options passed\nto `use/1` and `start_link/1`.\n\nApplication configuration is _not_ passed into the `init/2` callback. To use application\nconfig the callback must be overridden and merged manually.","ref":"Kiq.html#c:init/2"},{"type":"callback","title":"Kiq.start_link/1","doc":"Starts the client and possibly the supervision tree, returning `{:ok, pid}` when startup is\nsuccessful.\n\nReturns `{:error, {:already_started, pid}}` if the tree is already started or `{:error, term}`\nin case anything else goes wrong.","ref":"Kiq.html#c:start_link/1"},{"type":"callback","title":"Options - Kiq.start_link/1","doc":"Any options passed to `start_link` will be merged with those provided in the `use` block.","ref":"Kiq.html#c:start_link/1-options"},{"type":"type","title":"Kiq.job_args/0","doc":"","ref":"Kiq.html#t:job_args/0"},{"type":"type","title":"Kiq.job_opts/0","doc":"","ref":"Kiq.html#t:job_opts/0"},{"type":"module","title":"Kiq.Encoder","doc":"This module wraps `Jason.encode/1` to provide consistent and safe JSON encoding.\n\nTypically this module shouldn't be used outside of Kiq, but the function is\ndocumented for reference on how job arguments are encoded.","ref":"Kiq.Encoder.html"},{"type":"function","title":"Kiq.Encoder.encode/1","doc":"Safely encode terms into JSON, if possible.\n\nSome Elixir/Erlang types can't be represented as JSON. To make encoding as reliable as possible\nthe `encode/1` function attempts to convert incompatible terms into JSON friendly values.\n\nThe following sanitization is applied (recursively):\n\n* `struct` - Converted to a map using `Map.from_struct/1`\n* `tuple` — Converted to a list\n* `pid`, `port`, `reference` — Converted to a string using inspect, i.e. \"#PID \"","ref":"Kiq.Encoder.html#encode/1"},{"type":"module","title":"Kiq.Job","doc":"Used to construct a Sidekiq compatible job.\n\nThe job complies with the [Sidekiq Job Format][1], and contains the following\nfields:\n\n* `jid` - A 12 byte random number as a 24 character hex encoded string\n* `pid` — Process id of the worker running the job, defaults to the calling process\n* `class` - The worker class which is responsible for executing the job\n* `args` - The arguments passed which should be passed to the worker\n* `queue` - The queue where a job should be enqueued, defaults to \"default\"\n* `at` — A time at or after which a scheduled job should be performed, in Unix format\n* `created_at` - When the job was created, in Unix format\n* `enqueue_at` - When the job was enqueued, in Unix format\n\nRetry & Failure Fields:\n\n* `retry` - Tells the Kiq worker to retry the enqueue job\n* `retry_count` - The number of times we've retried so far\n* `failed_at` - The first time the job failed, in Unix format\n* `retried_at` — The last time the job was retried, in Unix format\n* `error_message` — The message from the last exception\n* `error_class` — The exception module (or class, in Sidekiq terms)\n* `backtrace` - The number of lines of error backtrace to store. Only present\n  for compatibility with Sidekiq, this field is ignored.\n\nEpiration Fields:\n\n* `expires_in` - How long to keep a job before expiring it and skipping\n  execution, in milliseconds\n\nUnique Fields:\n\n* `unique_for` - How long uniqueness will be enforced for a job, in\n  milliseconds\n* `unique_until` - Allows controlling when a unique lock will be removed,\n  valid options are \"start\" and \"success\".\n* `unlocks_at` - When the job will be unlocked, in milliseconds\n* `unique_token` - The uniqueness token calculated from class, queue and args\n\n[1]: https://github.com/mperham/sidekiq/wiki/Job-Format","ref":"Kiq.Job.html"},{"type":"function","title":"Kiq.Job.decode/1","doc":"Decode an encoded job from JSON into a Job struct.\n\nAll job keys are atomized except for those within arguments.\n\n# Example\n\n    iex> job = Kiq.Job.decode(~s({\"class\":\"MyWorker\",\"args\":[1,2]}))\n    ...> Map.take(job, [:class, :args])\n    %{class: \"MyWorker\", args: [1, 2]}\n\n    iex> job = Kiq.Job.decode(~s({\"class\":\"MyWorker\",\"args\":{\"a\":1}}))\n    ...> Map.get(job, :args)\n    %{\"a\" => 1}","ref":"Kiq.Job.html#decode/1"},{"type":"function","title":"Kiq.Job.encode/1","doc":"Encode a job as JSON.\n\nDuring the encoding process any keys with `nil` values are removed.","ref":"Kiq.Job.html#encode/1"},{"type":"function","title":"Kiq.Job.new/1","doc":"Build a new `Job` struct with all dynamic arguments populated.\n\n    iex> Kiq.Job.new(%{class: \"Worker\"}) |> Map.take([:class, :args, :queue])\n    %{class: \"Worker\", args: [], queue: \"default\"}\n\nTo fit more naturally with Elixir the `class` argument can be passed as `module`:\n\n    iex> Kiq.Job.new(module: \"Worker\").class\n    \"Worker\"\n\nOnly \"start\" and \"success\" are allowed as values for `unique_until`. Any\nother value will be nullified:\n\n    iex> Kiq.Job.new(class: \"A\", unique_until: \"start\").unique_until\n    \"start\"\n\n    iex> Kiq.Job.new(class: \"A\", unique_until: :start).unique_until\n    \"start\"\n\n    iex> Kiq.Job.new(class: \"A\", unique_until: \"whenever\").unique_until\n    nil","ref":"Kiq.Job.html#new/1"},{"type":"function","title":"Kiq.Job.to_map/1","doc":"Convert a job into a map suitable for encoding.\n\nFor Sidekiq compatibility and encodeability some values are rejected.\nSpecifically, the `retry_count` value is dropped when it is 0.","ref":"Kiq.Job.html#to_map/1"},{"type":"function","title":"Kiq.Job.to_module/1","doc":"Extract a fully qualified worker module from a job.\n\n# Examples\n\n    iex> Kiq.Job.to_module(Kiq.Job.new(module: \"Kiq.Job\"))\n    Kiq.Job","ref":"Kiq.Job.html#to_module/1"},{"type":"type","title":"Kiq.Job.t/0","doc":"","ref":"Kiq.Job.html#t:t/0"},{"type":"module","title":"Kiq.Periodic.Crontab","doc":"Generate and evaluate the structs used to evaluate periodic jobs.\n\nThe `Crontab` module provides parsing and evaluation for standard cron\nexpressions. Expressions are composed of rules specifying the minutes, hours,\ndays, months and weekdays. Rules for each field are comprised of literal\nvalues, wildcards, step values or ranges:\n\n  * `*` - Wildcard, matches any value (0, 1, 2, ...)\n  * `0` — Literal, matches only itself (only 0)\n  * `*/15` — Step, matches any value that is a multiple (0, 15, 30, 45)\n  * `0-5` — Range, matches any value within the range (0, 1, 2, 3, 4, 5)\n\nEach part may have multiple rules, where rules are separated by a comma. The\nallowed values for each field are as follows:\n\n  * `minute` - 0-59\n  * `hour` - 0-23\n  * `days` - 1-31\n  * `month` - 1-12 (or aliases, `JAN`, `FEB`, `MAR`, etc.)\n  * `weekdays` - 0-6 (or aliases, `SUN`, `MON`, `TUE`, etc.)\n\nFor more in depth information see the man documentation for `cron` and\n`crontab` in your system. Alternatively you can experiment with various\nexpressions online at [Crontab Guru](http://crontab.guru/).","ref":"Kiq.Periodic.Crontab.html"},{"type":"module","title":"Examples - Kiq.Periodic.Crontab","doc":"# The first minute of every hour\n    Crontab.parse!(\"0 * * * *\")\n\n    # Every fifteen minutes during standard business hours\n    Crontab.parse!(\"*/15 9-17 * * *\")\n\n    # Once a day at midnight during december\n    Crontab.parse!(\"0 0 * DEC *\")\n\n    # Once an hour during both rush hours on Friday the 13th\n    Crontab.parse!(\"0 7-9,4-6 13 * FRI\")","ref":"Kiq.Periodic.Crontab.html#module-examples"},{"type":"function","title":"Kiq.Periodic.Crontab.now?/2","doc":"Evaluate whether a crontab matches a datetime. The current datetime in UTC is\nused as the default.","ref":"Kiq.Periodic.Crontab.html#now?/2"},{"type":"function","title":"Examples - Kiq.Periodic.Crontab.now?/2","doc":"iex> Kiq.Periodic.Crontab.now?(%Crontab{})\n    true\n\n    iex> crontab = Crontab.parse!(\"* * * * *\")\n    ...> Kiq.Periodic.Crontab.now?(crontab)\n    true\n\n    iex> crontab = Crontab.parse!(\"59 23 1 1 6\")\n    ...> Kiq.Periodic.Crontab.now?(crontab)\n    false","ref":"Kiq.Periodic.Crontab.html#now?/2-examples"},{"type":"function","title":"Kiq.Periodic.Crontab.parse!/1","doc":"Parse a crontab expression string into a Crontab struct.","ref":"Kiq.Periodic.Crontab.html#parse!/1"},{"type":"function","title":"Examples - Kiq.Periodic.Crontab.parse!/1","doc":"iex> Kiq.Periodic.Crontab.parse!(\"0 6,12,18 * * *\")\n    %Crontab{minutes: [0], hours: [6, 12, 18]}\n\n    iex> Kiq.Periodic.Crontab.parse!(\"0-2,4-6 */12 * * *\")\n    %Crontab{minutes: [0, 1, 2, 4, 5, 6], hours: [0, 12]}\n\n    iex> Kiq.Periodic.Crontab.parse!(\"* * 20,21 SEP,NOV *\")\n    %Crontab{days: [20, 21], months: [9, 11]}\n\n    iex> Kiq.Periodic.Crontab.parse!(\"0 12 * * SUN\")\n    %Crontab{minutes: [0], hours: [12], weekdays: [0]}","ref":"Kiq.Periodic.Crontab.html#parse!/1-examples"},{"type":"type","title":"Kiq.Periodic.Crontab.expression/0","doc":"","ref":"Kiq.Periodic.Crontab.html#t:expression/0"},{"type":"type","title":"Kiq.Periodic.Crontab.t/0","doc":"","ref":"Kiq.Periodic.Crontab.html#t:t/0"},{"type":"module","title":"Kiq.Pool","doc":"Kiq maintains a fixed pool of Redix connections that are used by processes to\ncommunicate with Redis.\n\nThe pool is a fixed set of supervised connections. Calling `checkout/1` will\nreturn a random connection pid. The Redis connection is fully duplexed,\nmaking it safe for multiple processes to use the same connection\nsimultaneously. Connections never need to be checked back in.","ref":"Kiq.Pool.html"},{"type":"module","title":"Ad-Hoc Usage - Kiq.Pool","doc":"Each supervised Kiq instance will have its own pool. The pool name is derived\nfrom the module name, i.e. the module `MyApp.Kiq` would have a supervised\npool registered as `MyApp.Kiq.Pool`. The name can be used to checkout\nconnections and execute commands in the console.\n\nFor example, to get a list of the queues that are currently active:\n\n    MyApp.Kiq.Pool\n    |> Kiq.Pool.checkout()\n    |> Redix.command([\"SMEMBERS\", \"queues\"])","ref":"Kiq.Pool.html#module-ad-hoc-usage"},{"type":"function","title":"Kiq.Pool.checkout/1","doc":"Get the pid of a supervised Redix connection.\n\nConnections are randomly allocated and don't need to be checked back in.","ref":"Kiq.Pool.html#checkout/1"},{"type":"function","title":"Kiq.Pool.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Kiq.Pool.html#child_spec/1"},{"type":"type","title":"Kiq.Pool.options/0","doc":"","ref":"Kiq.Pool.html#t:options/0"},{"type":"behaviour","title":"Kiq.Reporter","doc":"Job handling is performed by reporters, which are customized `GenStage`\nconsumers.\n\nThis module specifies the reporter behaviour and provies an easy way to\ndefine new reporters via `use Kiq.Reporter`. The `using` macro defines all\nnecessary `GenStage` functions and defines no-op handlers for all events.","ref":"Kiq.Reporter.html"},{"type":"behaviour","title":"Example Custom Reporter - Kiq.Reporter","doc":"Custom reporters may be defined within your application. One common use-case\nfor custom reporters would be reporting exceptions to a tracking service. Here\nwe will define a reporter for [Honeybadger](https://honeybadger.io):\n\n    defmodule MyApp.Reporters.Honeybadger do\n      @moduledoc false\n\n      use Kiq.Reporter\n\n      @impl Kiq.Reporter\n      def handle_failure(job, error, stacktrace, state) do\n        metadata = Map.take(job, [:jid, :class, :args, :queue, :retry_count])\n\n        :ok = Honeybadger.notify(error, metadata, stacktrace)\n\n        state\n      end\n    end\n\nNext you'll need to specify the reporter inside your application's `Kiq`\nmodule, using `extra_reporters`:\n\n    defmodule MyApp.Kiq do\n      @moduledoc false\n\n      alias MyApp.Reporters.Honeybadger\n\n      use Kiq, queues: [default: 50], extra_reporters: [Honeybadger]\n    end\n\nThis guarantees that your reporter will be supervised by the reporter\nsupervisor, and that it will be re-attached to the reporter producer upon\nrestart if the reporter crashes.","ref":"Kiq.Reporter.html#module-example-custom-reporter"},{"type":"behaviour","title":"Notes About State - Kiq.Reporter","doc":"Handler functions are always invoked with the current state. The return value\nof each handler function will be used as the updated state after all events\nare processed.","ref":"Kiq.Reporter.html#module-notes-about-state"},{"type":"callback","title":"Kiq.Reporter.handle_aborted/3","doc":"Emitted when job processing has been intentionally aborted.","ref":"Kiq.Reporter.html#c:handle_aborted/3"},{"type":"callback","title":"Kiq.Reporter.handle_failure/4","doc":"Emitted when an exception ocurred while processing a job.","ref":"Kiq.Reporter.html#c:handle_failure/4"},{"type":"callback","title":"Kiq.Reporter.handle_started/2","doc":"Emitted when a worker starts processing a job.","ref":"Kiq.Reporter.html#c:handle_started/2"},{"type":"callback","title":"Kiq.Reporter.handle_stopped/2","doc":"Emitted when the worker has completed the job, regardless of whether it\nsucceded or failed.","ref":"Kiq.Reporter.html#c:handle_stopped/2"},{"type":"callback","title":"Kiq.Reporter.handle_success/3","doc":"Emitted when a worker has completed a job and no failures ocurred.","ref":"Kiq.Reporter.html#c:handle_success/3"},{"type":"type","title":"Kiq.Reporter.options/0","doc":"","ref":"Kiq.Reporter.html#t:options/0"},{"type":"type","title":"Kiq.Reporter.state/0","doc":"","ref":"Kiq.Reporter.html#t:state/0"},{"type":"module","title":"Kiq.Testing","doc":"This module simplifies making assertions about enqueued jobs during testing.\n\nTesting assertions only work when Kiq is started with `test_mode` set to\n`:sandbox`. In sandbox mode jobs are never flushed to Redis and are stored in\nmemory until the test run is over. Each enqueued job is associated with the\nprocess that enqueued it, allowing asynchronous tests to check stored jobs\nwithout any interference.","ref":"Kiq.Testing.html"},{"type":"module","title":"Using in Tests - Kiq.Testing","doc":"If your application has defined a top level Kiq module as `MyApp.Kiq`, then\nyou would `use` the testing module inside your application's case templates\nlike so:\n\n    use Kiq.Testing, client: MyApp.Kiq.Client\n\nThat will define two helper functions, `assert_enqueued/1` and\n`refute_enqueued/1`. The functions can then be used to make assertions on the\njobs that have been stored while testing.\n\nGiven a simple module that enqueues a job:\n\n    defmodule MyApp.Business do\n      alias MyApp.Kiq\n\n      def work(args) do\n        Kiq.enqueue(class: \"SomeWorker\", args: args)\n      end\n    end\n\nThe behaviour can be exercised in your test code:\n\n    defmodule MyApp.BusinessTest do\n      use ExUnit.Case, async: true\n      use Kiq.Testing, client: MyApp.Kiq.Client\n\n      alias MyApp.Business\n\n      test \"jobs are enqueued with provided arguments\" do\n        Business.work([1, 2])\n\n        assert_enqueued(class: \"SomeWorker\", args: [1, 2])\n      end\n    end","ref":"Kiq.Testing.html#module-using-in-tests"},{"type":"module","title":"Testing Jobs From Other Processes - Kiq.Testing","doc":"All calls to `assert_enqueued/3` and `refute_enqueued/3` use the `:sandbox`\nscope by default. That scope ensures that the current process can only find\nits own enqueued jobs. Sometimes this behavior is undesirable. For example,\nwhen jobs are being enqueued outside of the test process. If a separate\nserver or task enqueue a job you may use the `:shared` scoping to make global\nassertions.\n\n    Task.async(fn -> Kiq.enqueue(class: \"MyWorker\", args: [1, 2]) end)\n\n    assert_enqueued(:shared, class: \"MyWorker\")","ref":"Kiq.Testing.html#module-testing-jobs-from-other-processes"},{"type":"function","title":"Kiq.Testing.assert_enqueued/3","doc":"Assert that a job with particular options has been enqueued.\n\nOnly values for the provided arguments will be checked. For example, an\nassertion made on `class: \"MyWorker\"` will match _any_ jobs for that class,\nregardless of the args.","ref":"Kiq.Testing.html#assert_enqueued/3"},{"type":"function","title":"Kiq.Testing.refute_enqueued/3","doc":"Refute that a job with particular options has been enqueued.\n\nSee `assert_enqueued/2` for additional details.","ref":"Kiq.Testing.html#refute_enqueued/3"},{"type":"behaviour","title":"Kiq.Worker","doc":"Defines a behavior and macro to guide the creation of worker modules.\n\nWorker modules do the work of processing a job. At a minimum they must define\na `perform` function, which will be called with the arguments that were\nenqueued with the `Kiq.Job`.","ref":"Kiq.Worker.html"},{"type":"behaviour","title":"Defining Workers - Kiq.Worker","doc":"Define a worker to process jobs in the `events` queue:\n\n    defmodule MyApp.Workers.Business do\n      use Kiq.Worker, queue: \"events\", retry: 10, dead: false\n\n      @impl Kiq.Worker\n      def perform(args) do\n        IO.inspect(args)\n      end\n    end\n\nThe `perform/1` function will always receive a list of arguments. In this\nexample the worker will simply inspect any arguments that are provided.","ref":"Kiq.Worker.html#module-defining-workers"},{"type":"behaviour","title":"Enqueuing Jobs - Kiq.Worker","doc":"All workers implement a `new/1` function that converts a list of arguments\ninto a `Kiq.Job` that is suitable for enqueuing:\n\n    [\"doing\", \"business\"]\n    |> MyApp.Workers.Business.new()\n    |> MyApp.Kiq.enqueue()","ref":"Kiq.Worker.html#module-enqueuing-jobs"},{"type":"callback","title":"Kiq.Worker.new/1","doc":"Build a job for this worker using all default options.\n\nAny additional arguments that are provided will be merged into the job.","ref":"Kiq.Worker.html#c:new/1"},{"type":"callback","title":"Kiq.Worker.perform/1","doc":"The `perform/1` function is called with the enqueued arguments.\n\nThe return value is not important.","ref":"Kiq.Worker.html#c:perform/1"},{"type":"type","title":"Kiq.Worker.args/0","doc":"","ref":"Kiq.Worker.html#t:args/0"},{"type":"type","title":"Kiq.Worker.opts/0","doc":"","ref":"Kiq.Worker.html#t:opts/0"},{"type":"task","title":"mix kiq.gen.reporter","doc":"Generates a new reporter.\n\nThe module will be nested within the `lib` directory. Custom reporters\nmust be listed as `:extra_reporters` in your configuration.","ref":"Mix.Tasks.Kiq.Gen.Reporter.html"},{"type":"task","title":"Examples - mix kiq.gen.reporter","doc":"mix kiq.gen.reporter Kiq.Reporters.Custom","ref":"Mix.Tasks.Kiq.Gen.Reporter.html#module-examples"},{"type":"task","title":"mix kiq.gen.supervisor","doc":"Generates a new supervisor.\n\nThe module will be placed within the `lib` directory.","ref":"Mix.Tasks.Kiq.Gen.Supervisor.html"},{"type":"task","title":"Examples - mix kiq.gen.supervisor","doc":"mix kiq.gen.supervisor MyApp.Kiq","ref":"Mix.Tasks.Kiq.Gen.Supervisor.html#module-examples"},{"type":"task","title":"mix kiq.gen.worker","doc":"Generates a new worker.\n\nThe module will be nested within the `lib` directory.","ref":"Mix.Tasks.Kiq.Gen.Worker.html"},{"type":"task","title":"Examples - mix kiq.gen.worker","doc":"mix kiq.gen.worker MyApp.Workers.Business\n  mix kiq.gen.worker MyApp.Workers.Essential -q critical\n  mix kiq.gen.worker MyApp.Workers.Unimportant -r 3\n  mix kiq.gen.worker MyApp.Workers.Lifeless --no-dead","ref":"Mix.Tasks.Kiq.Gen.Worker.html#module-examples"},{"type":"task","title":"Command Line Options - mix kiq.gen.worker","doc":"* `-d`, `--dead`, `--no-dead` — whether the job will be pushed to the dead set when retries are exhausted\n  * `-q`, `--queue` — the queue this worker's jobs will run in\n  * `-r`, `--retry` — the number of times jobs will be retried","ref":"Mix.Tasks.Kiq.Gen.Worker.html#module-command-line-options"},{"type":"extras","title":"Kiq","doc":"# Kiq\n\n[![Build Status](https://travis-ci.org/sorentwo/kiq.svg?branch=master)](https://travis-ci.org/sorentwo/kiq)\n[![Hex.pm Version](http://img.shields.io/hexpm/v/kiq.svg)](https://hex.pm/packages/kiq)\n[![Hex.pm Downloads](https://img.shields.io/hexpm/dt/kiq.svg)](https://hex.pm/packages/kiq)\n[![Hex Docs](http://img.shields.io/badge/hex.pm-docs-green.svg?style=flat)](https://hexdocs.pm/kiq)\n\nKiq is a robust and extensible job processing queue that aims for compatibility\nwith [Sidekiq][sk], [Sidekiq Pro][skp] and [Sidekiq Enterprise][ske].\n\nJob queuing, processing and reporting are all built on [GenStage][genst]. That means\nmaximum parallelism with the safety of backpressure as jobs are processed.\n\n[genst]: https://github.com/elixir-lang/gen_stage\n\n#","ref":"readme.html"},{"type":"extras","title":"Why Kiq? - Kiq","doc":"Many features and architectural choices in Kiq were drawn from existing Elixir\njob processing packages like [Exq][exq], [Verk][verk] and [EctoJob][ej]. Each\nof those packages are great and have varying strenghts, but they lacked seamless\ninterop with Sidekiq Pro or Sidekiq Enterprise jobs.\n\nSidekiq Pro and Enterprise are amazing pieces of commercial software and worth\nevery cent—your organization should buy them! Kiq is intended as a _bridge_ for\nyour team to _interop_ between Ruby and Elixir. As an organization embraces\nElixir it becomes necessary to run some background jobs in Elixir, and it must\nbe just as reliable as when jobs were ran through Sidekiq.\n\n[sk]: https://sidekiq.org/\n[skp]: https://sidekiq.org/products/pro.html\n[ske]: https://sidekiq.org/products/enterprise.html\n[exq]: https://github.com/akira/exq\n[verk]: https://github.com/edgurgel/verk\n[ej]: https://github.com/mbuhot/ecto_job\n\n#","ref":"readme.html#why-kiq"},{"type":"extras","title":"Sidekiq Pro & Enterprise Comaptible Feature Set - Kiq","doc":"Kiq's feature set includes many marquee offerings from Sidekiq, Sidekiq Pro and\nSidekiq Enterprise—plus some additional niceties made possible by running on the\nBEAM. Here is a table highlighting the Kiq's features compared to the various\nSidekiq versions:\n\n| Feature            | Kiq         | Sidekiq    | Sidekiq Pro | Sidekiq Ent |\n| ------------------ | ----------- | ---------- | ----------- | ----------- |\n| Max Size Queues    | ✅          | ❌         | ❌          | ❌          |\n| Structured Logging | ✅          | ❌         | ❌          | ❌          |\n| Scheduled Jobs     | ✅          | ✅         | ✅          | ✅          |\n| Error Handling     | ✅          | ✅         | ✅          | ✅          |\n| Expiring Jobs      | ✅          | ❌         | ✅          | ✅          |\n| Worker Metrics     | ✅          | ❌         | ✅          | ✅          |\n| Reliable Client    | ✅          | ❌         | ✅          | ✅          |\n| Reliable Server    | ✅          | ❌         | ✅          | ✅          |\n| Rolling Restarts   | ✅          | ❌         | ❌          | ✅          |\n| Periodic Jobs      | ✅          | ❌         | ❌          | ✅          |\n| Unique Jobs        | ✅          | ❌         | ❌          | ✅          |\n| Leader Election    | ✅          | ❌         | ❌          | ✅          |\n| Multi Process      | ✅          | ❌         | ❌          | ✅          |\n| Web UI †           | ❌          | ✅         | ✅          | ✅          |\n| Batch Jobs ‡       | ❌          | ❌         | ✅          | ✅          |\n| Encryption ‡       | ❌          | ❌         | ❌          | ✅          |\n| Rate Limiting ‡    | ❌          | ❌         | ❌          | ✅          |\n\n* † Kiq relies on Sidekiq's Web UI\n* ‡ Planned, but not implemented yet\n\nIf a feature isn't supported or planned it is _probably_ for one of these\nreasons:\n\n1. We get it for free on the BEAM and it isn't necessary, i.e. (safe shutdown,\n   multi-process, rolling restarts)\n3. We enable developers to use custom reporters to do it themselves (stats,\n   error reporting)\n\n#","ref":"readme.html#sidekiq-pro-enterprise-comaptible-feature-set"},{"type":"extras","title":"Design Decisions - Kiq","doc":"* Avoid global and compile time configuration. All configuration can be defined\n  programatically, eliminating the need for hacks like `{:system, \"REDIS_URL\"}`.\n* Not an application, it is included in your application's supervision tree\n* Testing focused, provide helpers and modes to aid testing\n* Extensible job handling via GenStage consumers\n* Simplified worker definitions to ease job definition and pipelining\n\n[ent]: https://sidekiq.org/products/enterprise.html","ref":"readme.html#design-decisions"},{"type":"extras","title":"Installation - Kiq","doc":"Add `kiq` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:kiq, \"~> 0.7\"}\n  ]\nend\n```\n\nThen run `mix deps.get` to install the dependency.\n\n\nFinally, add the supervisor to your application's supervision tree:\n\n```elixir\n{MyApp.Kiq, []}\n```\n\nKiq itself is not an application and must be started within your application's\nsupervision tree. All of your application's configuration and custom methods\nshould be put into the supervisor.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Kiq","doc":"Kiq isn't an application that must be started. Similarly to Ecto, you define\none or more Kiq modules within your application. This allows multiple\nsupervision trees with entirely different configurations.\n\nRun the generator to define a Kiq supervisor for your application:\n\n```bash\nmix kiq.gen.supervisor MyApp.Kiq\n```\n\nInclude the module in your application's supervision tree:\n\n```elixir\ndefmodule MyApp.Application do\n  @moduledoc false\n\n  use Application\n\n  alias MyApp.{Endpoint, Kiq, Repo}\n\n  def start(_type, _args) do\n    children = [\n      {Repo, []},\n      {Endpoint, []},\n      {Kiq, []}\n    ]\n\n    Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n  end\nend\n```\n\nWith the supervision tree in place you are ready to start creating workers! The\nsimplest way to create a worker is through the generator:\n\n```bash\nmix kiq.gen.worker MyApp.Workers.Business\n```\n\nThat will define a worker with a `perform/1` function where all the magic will\nhappen.\n\nSee `mix help kiq.gen.worker` for additional options.\n\nCheck the [hexdocs][hd] for additional details, configuration options, how to\ntest, defining workers and custom reporters.\n\n[hd]: https://hexdocs.pm/kiq","ref":"readme.html#usage"},{"type":"extras","title":"Benchmarks - Kiq","doc":"Kiq has a set of benchmarks to track the performance of important operations.\nBenchmarks are ran using the [Benchee][benchee] library and require Redis to be\nrunning.\n\nTo run all benchmarks:\n\n```bash\nmix run bench/bench_helper.exs\n```\n\n[benchee]: https://github.com/PragTob/benchee","ref":"readme.html#benchmarks"},{"type":"extras","title":"Contributing - Kiq","doc":"Clone the repository and run `$ mix test` to make sure everything is working. For\ntests to pass, you must have a Redis server running on `localhost`, port `6379`,\ndatabase `3`. You can configure a different host, port and database by setting\nthe `REDIS_URL` environment variable before testing.\n\nNote that tests will wipe the the configured database on the Redis server\nmultiple times while testing. By default database 3 is used for testing.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Kiq","doc":"Kiq is released under the MIT license. See the [LICENSE](LICENSE.txt).","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](http://keepachangelog.com/en/1.0.0/)\nand this project adheres to [Semantic Versioning](http://semver.org/spec/v2.0.0.html).\n\n## [0.7.3] — 2023-07-18\n\n- Update dependency constraints for `telemetry`, `nimble_parsec`, `redix`, and `gen_stage`\n\n## [0.7.2] — 2021-05-12\n\n- Update and loosen dependency constraints\n\n## [0.7.1] — 2020-05-06\n\n- Update and loosen dependency constraints, particularly for nimble parsec\n\n## [0.7.0] - 2019-05-13\n\n#","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- [Reporter.Instrumenter] Bump telemetry version.\n\n## [0.6.0] - 2019-03-18\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- [Kiq.Encoder] Wrap `Jason.encode` and attempt to sanitize terms that can't be\n  represented as JSON. This is primarily a safeguard against enqueuing jobs with\n  structs or tuples as arguments.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- [Kiq.Client.Resurrection] Correct usage of SCAN to ensure all backup queues\n  will be found for re-enqueueing.\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Changed - Changelog","doc":"- [Kiq.Job] Keys within job arguments are no longer converted to atoms. This is\n  for safe execution and is in line with how Phoenix passes controller\n  arguments.\n- [Kiq.Client] Enqueue all jobs in a single pipelined command. This moves all of\n  the unique, scheduling and enqueueing logic into a lua script. The script\n  reduces the number of round trips to redis and enabled pipelining. Overall\n  this improved job enqueue/execute performance from 1800 jobs/sec to slightly\n  more than 3700 jobs/sec.\n\n## [0.5.0] — 2018-12-14\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- [Kiq.Periodic] Periodic jobs support! Periodic (AKA \"Cron\") jobs are\n  registered with a schedule and enqueued automatically by Kiq accordingly.\n  This is a fully fledged alternative to using a separate scheduler such as\n  [Quantum][quantum].\n\n[quantum]: https://github.com/quantum-elixir/quantum-core\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- [Kiq.Reporter.Logger] Job lifecycle logging uses the format `event:\n  \"job_status\"` instead of `status: \"status\"`. For example, if you were checking\n  logs for `\"status\":\"finished\"` you would now look for\n  `\"event\":\"job_finished\"`.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- [Kiq.Client] Eliminate use of `MUTLI/EXEC` for stats and queuing. This caused\n  a race condition that could prevent stats being recorded and jobs from being\n  enqueued.\n\n## [0.4.0] — 2018-12-03\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- [Reporter.Retryer] Dead jobs support! Jobs with exhausted retries are moved\n  into the `dead` set. Workers or jobs may be configured with `dead: false` to\n  prevent being moved to the dead set.\n\n- [Kiq] Quiet support! The `configure/1` function can be used to set runtime\n  configuration. Calling `MyKiq.configure(quiet: true)` will stop all queues\n  from starting new jobs while allowing currently running jobs to finish. This\n  can be used for graceful shutdown and smooth blue/green deployment.\n\n- [Reporter.Instrumenter] A new reporter that provides instrumentation data\n  through [Telemetry](https://hexdocs.pm/telemetry). Reporting job execution\n  metrics only requires attaching to `[:kiq, :job, event]` events.\n\n- [mix kiq.gen.supervisor] A new generator to create an initial Kiq supervisor\n  module.\n\n- [mix kiq.gen.reporter] A new generator to create custom reporters with all of\n  the callbacks defined.\n\n- [mix kiq.gen.worker] A new generator to create workers with `retries`, `queue`\n  and `dead` specified.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- [Script] More functionality, including dequeueing and descheduling, has been\n  moved into Lua scripts. To reduce bandwidth and io overhead all `EVAL` usage\n  has been replaced with `EVALSHA`.\n\n- [Identity] For purely aesthetic reasons the nonce value is now lower case.\n\n- [Logger] Consistently format logs as JSON. Previously some ad-hoc logs were\n  unstructured, which made parsing difficult. Additionally, logs now contain\n  `\"source\":\"kiq\"` for easier identification.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- [Kiq] In progress backup queues, AKA \"Private Queues\", are now implemented\n  with a hash rather than a list. This ensures that a job can _always_ be\n  removed from the backup, regardless of how it is encoded. Previously, when a\n  job was enqueued by Sidekiq the serialized version wouldn't match the\n  version serialized by Kiq. The `LREM` command requires an exact binary match\n  or list elements won't be removed, which caused jobs to linger in the backup\n  queue.\n\n## [0.3.0] — 2018-10-28\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- [Kiq] Expiring Job support. Expiring jobs won't be ran after a configurable\n  amount of time. The expiration period is set with the `expires_in` option,\n  which accepts millisecond values identically to the `unique_for` option.\n\n- [Kiq.Pool] Connection pooling. Recently added benchmarks quickly identified a\n  bottleneck in the single Client/Redis connection. A simple random-order pool\n  was introduced that allowed the client, queues and reporters to rotate through\n  a set of connections. This change alone doubled the enqueue/dequeue\n  throughput.\n\n- [Kiq.Client] Reliable push support. Enqueued jobs are now buffered in memory\n  and periodically flushed to Redis. If there are any connection errors or Redis\n  is down the jobs are retained and flushing is retried with backoff.\n\n- [Kiq.Testing] Sandbox test mode. When `test_mode` is set to `:sandbox` jobs\n  will never be flushed to Redis. Each locally buffered job is associated with\n  the process that enqueued it, enabling concurrent testing with isolation\n  between test runs.\n\n- [Kiq] Leadership election. Useful when coordinating work that should only\n  happen on a single node. Internally used to prevent duplicate job\n  resurrection, and in the future will be used for periodic jobs.\n\n- [Kiq] Private queues. Ensure terminated jobs are resurrected when the\n  application starts up. Unlike the previous job backup mechanism this\n  guarantees that only jobs from dead processors are resurrected; in-process\n  jobs will never be duplicated.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- [Redix] Upgrade from Redix 0.7.X to 0.8.X, which introduced the `noreply_` variant of\n  commands.\n\n- [Kiq.Config] Renamed `poll_interval` option to `flush_interval`, and the\n  default changed from `1000` to `500`.\n\n#","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- [Kiq] Failing unique jobs would never be retried.\n\n## [0.2.0] — 2018-09-17\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"- [Kiq] Unique Job support! Workers, and their corresponding jobs, can now be\n  made unique within for a period of time. Until the initial job has succeeded\n  it is considered \"locked\" and all subsequent attempts to enqueue a job with\n  the same class, queue and args will be ignored. The unique period is specified\n  with the `unique_for` option, which accepts millisecond values for easy\n  interaction with the `:timer` module's `minutes/1` and `seconds/1` functions.\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixed - Changelog","doc":"- [Kiq.Runner] Correctly convert nested modules binaries into the correct\n  worker, i.e.  convert `\"Kiq.Workers.Business\"` into the module\n  `Kiq.Workers.Business`.\n\n- [Kiq.Job] Correctly serialize pid and job payloads when recording a worker's\n  running jobs for heartbeats. Incorrectly formatted hashes in Redis will cause\n  the Sidekiq \"busy\" dashboard to crash.\n\n- [Kiq.Heartbeat] Retain the original `ran_at` value for reported jobs. Running\n  jobs are now encoded when they are started, preventing repeated JSON encodings\n  and allowing the initial `ran_at` value to be used when reporting. Prior to\n  this change the job was always reported as being started \"Just Now\".\n\n- [Kiq.Job] Skip serializing `retry_count` when the value is 0. Sidekiq doesn't\n  include a `retry_count` when there hasn't been a retry, which prevents the job\n  serialized by Kiq from matching up. If the serialized job doesn't match up\n  then the job can't be removed from the backup queue, leading to a buildup of\n  duplicate jobs.\n\n## [0.1.0] - 2018-07-31\n\n#","ref":"changelog.html#fixed"},{"type":"extras","title":"Added - Changelog","doc":"Initial release, everything was added!\n\n- Job creation\n- Worker definition\n- Running and configuring queues\n- Retries\n- Scheduled jobs\n- Statistics\n- Custom reporters\n- Testing support\n- Heartbeat reporting\n\n[Unreleased]: https://github.com/sorentwo/kiq/compare/v0.1.0...HEAD\n[0.1.0]: https://github.com/sorentwo/kiq/compare/e6106af2506...v0.1.0\n[0.2.0]: https://github.com/sorentwo/kiq/compare/v0.1.0...v0.2.0\n[0.3.0]: https://github.com/sorentwo/kiq/compare/v0.2.0...v0.3.0\n[0.4.0]: https://github.com/sorentwo/kiq/compare/v0.3.0...v0.4.0\n[0.5.0]: https://github.com/sorentwo/kiq/compare/v0.4.0...v0.5.0","ref":"changelog.html#added"}],"content_type":"text/markdown"}