searchNodes=[{"doc":"Kiq is a robust and extensible job processing queue that aims for compatibility with Sidekiq Enterprise. Job queuing, processing and reporting are all built on GenStage. That means maximum parallelism with the safety of backpressure as jobs are processed. Usage Kiq isn&#39;t an application that must be started. Similarly to Ecto, you define one or more Kiq modules within your application. This allows multiple supervision trees with entirely different configurations. Run the generator to create a new Kiq supervisor for your application: mix kiq.gen.supervisor -m MyApp.Kiq Alternatively, manually define a Kiq module for your application: defmodule MyApp.Kiq do use Kiq, queues: [default: 25, events: 50] end Include the module in your application&#39;s supervision tree: defmodule MyApp.Application do @moduledoc false use Application alias MyApp.{Endpoint, Kiq, Repo} def start(_type, _args) do children = [ {Repo, []}, {Endpoint, []}, {Kiq, []} ] Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor) end end Configuration Kiq is used to start one or more supervision trees in your application. That means there isn&#39;t a central Kiq &quot;app&quot; to configure, instead each supervision tree may be configured independently. Configuration options pass through a couple of locations, accumulating overrides until finally passing through the init/2 callback. Options are accumulated in this order: Options passed into via the use macro. These should be constant compile time options, i.e. extra_reporters. Options passed to start_link/1 by the application&#39;s supervision tree. These should also be values suitable for compile time. Injected by the init/2 callback inside your application&#39;s Kiq instance. This is where runtime configuration such as the Redis URL or environment specific options should be passed. The options can come from any dynamic source such as Mix Config, Vault, Etcd, environment variables, etc. The default init/2 implementation uses System.get_env/1 to read the REDIS_URL on boot. The default callback can be overridden to pull in additional configuration. For example, to grab values from Application.get_env/2: def init(_reason, opts) do for_env = Application.get_env(:my_app, :kiq, []) opts = opts |&gt; Keyword.merge(for_env) |&gt; Keyword.put(:client_opts, [redis_url: System.get_env(&quot;REDIS_URL&quot;)]) {:ok, opts} end The opts argument contains all configuration from stages 1 and 2 (the use macro and the call to start_link/1). Supervisor Configuration These configuration options must be provided to the supervision tree on startup: :client_opts — A keyword list of options passed to each Redix connection. This must contain the key :redis_url, which is passed to Redix.start_link/1 as the first argument. :dead_limit — The maximum number of jobs that will be retained in the dead set. Jobs beyond the limit are pruned any time a new job is moved to the dead set. The default is 10_000. :dead_timeout — The maximum amount of time that a job will remain in the dead set before being purged, specified in seconds. The default is 6 months. :extra_reporters — Additional reporters that your application will use to report errors, track external stats, etc. :fetch_interval — How frequently to poll for new jobs. Polling only happens when consumers aren&#39;t actively requesting new jobs. :flush_interval - How frequently locally enqueued jobs will be pushed to Redis. This defaults to 10ms, though it will back-off by a factor of 1.5 if there are any connection errors. :periodics — A list of job scheduling tuples in the form {schedule, worker} or {schedule, worker, options}. See Periodic Jobs for details. :pool_size — Controls the number of Redis connections available to Kiq, defaults to 5. :queues — A keyword list of queues where each entry is the name of the queue and the concurrency setting. For example, setting [default: 10, exports: 5, media: 5] would start the queues default, exports and media with a combined concurrency of 20. The concurrency setting specifies how many jobs each queue will run concurrently. :schedulers — A list of schedulers to run. The default schedulers are &quot;retry&quot; which is used to retry failed jobs with a backoff and &quot;schedule&quot;, used to enqueue jobs at a specific time in the future. Set this to an empty list to disable all schedulers and allow Sidekiq to handle enqueuing retries and scheduled jobs. :server? — Whether to start the queue supervisors and start processing jobs or only start the client. This setting is useful for testing or deploying your application&#39;s web and workers separately. :test_mode — Either :disabled or :sandbox. See Testing for details. Runtime Configuration These configuration options may be provided when starting a Kiq supervisor, but may also be set dynamically at runtime: :quiet — Instruct each queue to stop processing new jobs. Any in-progress jobs will keep running uninterrupted. For example, to quiet all queue producers: MyKiq.configure(quiet: true) Testing Kiq has special considerations to facilitate isolated and asynchronous testing. For testing Kiq should be configured in :sandbox mode and have the server disabled. This can be done specifically for the test environment by adding config to config/test.exs: config :my_app, :kiq, server?: false, test_mode: :sandbox Running in :sandbox mode ensures that enqueued jobs stay in memory and are never flushed to Redis. This allows your tests to use Kiq.Testing to make quick assertions about which jobs have or haven&#39;t been enqueued. See the docs for Kiq.Testing for more details and usage. Reliable Push Reliable push replicates the safety aspects of Sidekiq Pro&#39;s reliability client. To guard against network errors or other Redis issues the client buffers all jobs locally. At a frequent interval the jobs are flushed to Redis. If there are any errors while flushing the jobs will remain in memory until flushing can be retried later. Caveats The local job buffer is stored in memory, if the server is restarted suddently some jobs may be lost. There isn&#39;t any limit on the number of jobs that can be buffered. However, to conserve space jobs are stored compressed. Private Queues Kiq tries to prevent all job loss through private queues, a variant of the Super Fetch mechanism available in Sidekiq Pro. When jobs are executed they are backed up to a private queue specific to the server processing the job. If the processor crashes or the application is terminated before the job is finished the jobs remain backed up. On startup, and periodically afterward, jobs in any dead private queues are pushed back to the public queue for re-execution. This solution is well suited to containerized environments and autoscaling. Unique Jobs Kiq supports Sidekiq Enterprise&#39;s unique jobs. This feature prevents enqueueing duplicate jobs while an original job is still pending. The operations that attempt to enforce uniqueness are not atomic—uniquess is not guaranteed and should be considered best effort. Enable unique jobs for a worker by setting a unique time period: use Kiq.Worker, unique_for: :timer.minutes(5) Unlock Policy By default unique jobs will hold the unique lock until the job has ran successfully. This policy ensures that jobs will remain locked if there are any errors, but it is prone to race conditions in certain situations for longer running jobs. Generally it is best to stick with the default :success policy. However, if your job is effected by the race condition you can change your worker&#39;s policy: use Kiq.Worker, unique_for: :timer.minutes(60), unique_until: :start Caveats Note that job uniqueness is calculated from the class, args, and queue. This means that jobs with identical args may be added to different queues. Unique jobs enqueued by Sidekiq will be unlocked by Kiq, but they may not use the same lock value. This is due to differences between hashing Erlang terms and Ruby objects. To help ensure uniqueness always enqueue unique jobs from either Sidekiq or Kiq. Expiring Jobs Kiq supports Sidekiq Pro&#39;s expiring jobs. Expiring jobs won&#39;t be ran after a configurable amount of time. The expiration period is set with the expires_in option, which accepts millisecond values identically unique_for. Set the expiration for a worker using a relative time: use Kiq.Worker, expires_in: :timer.hours(1) Expiration time applies after the scheduled time. That means scheduling a job to run in an hour, with an expiration of 30 minutes, will expire in one hour and 30 minutes. Periodic Jobs Kiq supports Sidekiq Enterprise&#39;s Periodic Jobs. This allows jobs to be registered with a schedule and enqueued automatically. Jobs are registered as {crontab, worker} or {crontab, worker, options} using the :periodics attribute: use Kiq, periodics: [ {&quot;* * * * *&quot;, MyApp.MinuteWorker}, {&quot;0 * * * *&quot;, MyApp.HourlyWorker}, {&quot;0 0 * * *&quot;, MyApp.DailyWorker, retry: 1}, ] These jobs would be executed as follows: MyApp.MinuteWorker - Executed once every minute MyApp.HourlyWorker - Executed at the first minute of every hour MyApp.DailyWorker - Executed at midnight every day The crontab format, as parsed by Kiq.Parser.Crontab, respects all standard rules and has one minute resolution. That means it isn&#39;t possible to enqueue a job ever N seconds. Caveats All schedules are evaluated as UTC, the local timezone is never taken into account. Periodic jobs registered in Kiq aren&#39;t visible in the Loop panel of the Sidekiq Dashboard. This is due to the way loop data is stored by Sidekiq and can&#39;t be worked around. This is an alternative to using using a separate scheduler such as Quantum. However, unlike Quantum, Kiq doesn&#39;t support node based clustering, instead it uses Redis to coordinate and distrubte work. This means workers can scale horizontally even in a restricted environment like Heroku. Instrumentation &amp; Metrics The instrumentation reporter provides integration with Telemetry, a dispatching library for metrics. It is easy to report Kiq metrics to any backend by attaching to :kiq events. For exmaple, to log out the timing for all successful jobs: defmodule KiqJobLogger do require Logger def handle_event([:kiq, :job, :success], timing, metadata, _config) do Logger.info(&quot;[\#{metadata.queue}] \#{metadata.class} finished in \#{timing}&quot;) end end :telemetry.attach([:kiq, :job, :success], &amp;KiqJobLogger.handle_event/4, nil) Here is a reference for the available metrics: eventnamevaluemetadata started[:kiq, :job, :started]1:class, :queue success[:kiq, :job, :success]timing:class, :queue aborted[:kiq, :job, :aborted]1:class, :queue, :reason failure[:kiq, :job, :failure]1:class, :queue, :error","ref":"Kiq.html","title":"Kiq","type":"behaviour"},{"doc":"Clear all enqueued, scheduled and backup jobs. All known queues are cleared, even if they aren&#39;t listed in the current configuration.","ref":"Kiq.html#c:clear/0","title":"Kiq.clear/0","type":"callback"},{"doc":"Set runtime configuration values. See the &quot;Runtime Configuration&quot; section in the Kiq module documentation.","ref":"Kiq.html#c:configure/1","title":"Kiq.configure/1","type":"callback"},{"doc":"Enqueue a job to be processed asynchronously. Jobs can be enqueued from Job structs, maps or keyword lists. Options in - The amount of time in seconds to wait before processing the job. This must be a positive integer. at - A specific DateTime in the future when the job should be processed. Examples # Enqueue a job to be processed immediately MyJob.new([1, 2]) |&gt; MyKiq.enqueue() # Enqueue a job in one minute MyJob.new([1, 2]) |&gt; MyKiq.enqueue(in: 60) # Enqueue a job some time in the future MyJob.new([1, 2]) |&gt; MyKiq.enqueue(at: ~D[2020-09-20 12:00:00]) # Enqueue a job from scratch, without using a worker module MyKiq.enqueue(class: &quot;ExternalWorker&quot;, args: [1])","ref":"Kiq.html#c:enqueue/2","title":"Kiq.enqueue/2","type":"callback"},{"doc":"A callback executed when the supervision tree is started and possibly when configuration is read. The first argument is the context of the callback being invoked. In most circumstances this will be :supervisor. The second argument is a keyword list of the combined options passed to use/1 and start_link/1. Application configuration is not passed into the init/2 callback. To use application config the callback must be overridden and merged manually.","ref":"Kiq.html#c:init/2","title":"Kiq.init/2","type":"callback"},{"doc":"Starts the client and possibly the supervision tree, returning {:ok, pid} when startup is successful. Returns {:error, {:already_started, pid}} if the tree is already started or {:error, term} in case anything else goes wrong. Options Any options passed to start_link will be merged with those provided in the use block.","ref":"Kiq.html#c:start_link/1","title":"Kiq.start_link/1","type":"callback"},{"doc":"","ref":"Kiq.html#t:job_args/0","title":"Kiq.job_args/0","type":"type"},{"doc":"","ref":"Kiq.html#t:job_opts/0","title":"Kiq.job_opts/0","type":"type"},{"doc":"This module wraps Jason.encode/1 to provide consistent and safe JSON encoding. Typically this module shouldn&#39;t be used outside of Kiq, but the function is documented for reference on how job arguments are encoded.","ref":"Kiq.Encoder.html","title":"Kiq.Encoder","type":"module"},{"doc":"Safely encode terms into JSON, if possible. Some Elixir/Erlang types can&#39;t be represented as JSON. To make encoding as reliable as possible the encode/1 function attempts to convert incompatible terms into JSON friendly values. The following sanitization is applied (recursively): struct - Converted to a map using Map.from_struct/1 tuple — Converted to a list pid, port, reference — Converted to a string using inspect, i.e. &quot;#PID&quot;","ref":"Kiq.Encoder.html#encode/1","title":"Kiq.Encoder.encode/1","type":"function"},{"doc":"Used to construct a Sidekiq compatible job. The job complies with the Sidekiq Job Format, and contains the following fields: jid - A 12 byte random number as a 24 character hex encoded string pid — Process id of the worker running the job, defaults to the calling process class - The worker class which is responsible for executing the job args - The arguments passed which should be passed to the worker queue - The queue where a job should be enqueued, defaults to &quot;default&quot; at — A time at or after which a scheduled job should be performed, in Unix format created_at - When the job was created, in Unix format enqueue_at - When the job was enqueued, in Unix format Retry &amp; Failure Fields: retry - Tells the Kiq worker to retry the enqueue job retry_count - The number of times we&#39;ve retried so far failed_at - The first time the job failed, in Unix format retried_at — The last time the job was retried, in Unix format error_message — The message from the last exception error_class — The exception module (or class, in Sidekiq terms) backtrace - The number of lines of error backtrace to store. Only present for compatibility with Sidekiq, this field is ignored. Epiration Fields: expires_in - How long to keep a job before expiring it and skipping execution, in milliseconds Unique Fields: unique_for - How long uniqueness will be enforced for a job, in milliseconds unique_until - Allows controlling when a unique lock will be removed, valid options are &quot;start&quot; and &quot;success&quot;. unlocks_at - When the job will be unlocked, in milliseconds unique_token - The uniqueness token calculated from class, queue and args","ref":"Kiq.Job.html","title":"Kiq.Job","type":"module"},{"doc":"Decode an encoded job from JSON into a Job struct. All job keys are atomized except for those within arguments. Example iex&gt; job = Kiq.Job.decode(~s({&quot;class&quot;:&quot;MyWorker&quot;,&quot;args&quot;:[1,2]})) ...&gt; Map.take(job, [:class, :args]) %{class: &quot;MyWorker&quot;, args: [1, 2]} iex&gt; job = Kiq.Job.decode(~s({&quot;class&quot;:&quot;MyWorker&quot;,&quot;args&quot;:{&quot;a&quot;:1}})) ...&gt; Map.get(job, :args) %{&quot;a&quot; =&gt; 1}","ref":"Kiq.Job.html#decode/1","title":"Kiq.Job.decode/1","type":"function"},{"doc":"Encode a job as JSON. During the encoding process any keys with nil values are removed.","ref":"Kiq.Job.html#encode/1","title":"Kiq.Job.encode/1","type":"function"},{"doc":"Build a new Job struct with all dynamic arguments populated. iex&gt; Kiq.Job.new(%{class: &quot;Worker&quot;}) |&gt; Map.take([:class, :args, :queue]) %{class: &quot;Worker&quot;, args: [], queue: &quot;default&quot;} To fit more naturally with Elixir the class argument can be passed as module: iex&gt; Kiq.Job.new(module: &quot;Worker&quot;).class &quot;Worker&quot; Only &quot;start&quot; and &quot;success&quot; are allowed as values for unique_until. Any other value will be nullified: iex&gt; Kiq.Job.new(class: &quot;A&quot;, unique_until: &quot;start&quot;).unique_until &quot;start&quot; iex&gt; Kiq.Job.new(class: &quot;A&quot;, unique_until: :start).unique_until &quot;start&quot; iex&gt; Kiq.Job.new(class: &quot;A&quot;, unique_until: &quot;whenever&quot;).unique_until nil","ref":"Kiq.Job.html#new/1","title":"Kiq.Job.new/1","type":"function"},{"doc":"Convert a job into a map suitable for encoding. For Sidekiq compatibility and encodeability some values are rejected. Specifically, the retry_count value is dropped when it is 0.","ref":"Kiq.Job.html#to_map/1","title":"Kiq.Job.to_map/1","type":"function"},{"doc":"Extract a fully qualified worker module from a job. Examples iex&gt; Kiq.Job.to_module(Kiq.Job.new(module: &quot;Kiq.Job&quot;)) Kiq.Job","ref":"Kiq.Job.html#to_module/1","title":"Kiq.Job.to_module/1","type":"function"},{"doc":"","ref":"Kiq.Job.html#t:t/0","title":"Kiq.Job.t/0","type":"type"},{"doc":"Generate and evaluate the structs used to evaluate periodic jobs. The Crontab module provides parsing and evaluation for standard cron expressions. Expressions are composed of rules specifying the minutes, hours, days, months and weekdays. Rules for each field are comprised of literal values, wildcards, step values or ranges: * - Wildcard, matches any value (0, 1, 2, ...) 0 — Literal, matches only itself (only 0) */15 — Step, matches any value that is a multiple (0, 15, 30, 45) 0-5 — Range, matches any value within the range (0, 1, 2, 3, 4, 5) Each part may have multiple rules, where rules are separated by a comma. The allowed values for each field are as follows: minute - 0-59 hour - 0-23 days - 1-31 month - 1-12 (or aliases, JAN, FEB, MAR, etc.) weekdays - 0-6 (or aliases, SUN, MON, TUE, etc.) For more in depth information see the man documentation for cron and crontab in your system. Alternatively you can experiment with various expressions online at Crontab Guru. Examples # The first minute of every hour Crontab.parse!(&quot;0 * * * *&quot;) # Every fifteen minutes during standard business hours Crontab.parse!(&quot;*/15 9-17 * * *&quot;) # Once a day at midnight during december Crontab.parse!(&quot;0 0 * DEC *&quot;) # Once an hour during both rush hours on Friday the 13th Crontab.parse!(&quot;0 7-9,4-6 13 * FRI&quot;)","ref":"Kiq.Periodic.Crontab.html","title":"Kiq.Periodic.Crontab","type":"module"},{"doc":"Evaluate whether a crontab matches a datetime. The current datetime in UTC is used as the default. Examples iex&gt; Kiq.Periodic.Crontab.now?(%Crontab{}) true iex&gt; crontab = Crontab.parse!(&quot;* * * * *&quot;) ...&gt; Kiq.Periodic.Crontab.now?(crontab) true iex&gt; crontab = Crontab.parse!(&quot;59 23 1 1 6&quot;) ...&gt; Kiq.Periodic.Crontab.now?(crontab) false","ref":"Kiq.Periodic.Crontab.html#now?/2","title":"Kiq.Periodic.Crontab.now?/2","type":"function"},{"doc":"Parse a crontab expression string into a Crontab struct. Examples iex&gt; Kiq.Periodic.Crontab.parse!(&quot;0 6,12,18 * * *&quot;) %Crontab{minutes: [0], hours: [6, 12, 18]} iex&gt; Kiq.Periodic.Crontab.parse!(&quot;0-2,4-6 */12 * * *&quot;) %Crontab{minutes: [0, 1, 2, 4, 5, 6], hours: [0, 12]} iex&gt; Kiq.Periodic.Crontab.parse!(&quot;* * 20,21 SEP,NOV *&quot;) %Crontab{days: [20, 21], months: [9, 11]} iex&gt; Kiq.Periodic.Crontab.parse!(&quot;0 12 * * SUN&quot;) %Crontab{minutes: [0], hours: [12], weekdays: [0]}","ref":"Kiq.Periodic.Crontab.html#parse!/1","title":"Kiq.Periodic.Crontab.parse!/1","type":"function"},{"doc":"","ref":"Kiq.Periodic.Crontab.html#t:expression/0","title":"Kiq.Periodic.Crontab.expression/0","type":"type"},{"doc":"","ref":"Kiq.Periodic.Crontab.html#t:t/0","title":"Kiq.Periodic.Crontab.t/0","type":"type"},{"doc":"Kiq maintains a fixed pool of Redix connections that are used by processes to communicate with Redis. The pool is a fixed set of supervised connections. Calling checkout/1 will return a random connection pid. The Redis connection is fully duplexed, making it safe for multiple processes to use the same connection simultaneously. Connections never need to be checked back in. Ad-Hoc Usage Each supervised Kiq instance will have its own pool. The pool name is derived from the module name, i.e. the module MyApp.Kiq would have a supervised pool registered as MyApp.Kiq.Pool. The name can be used to checkout connections and execute commands in the console. For example, to get a list of the queues that are currently active: MyApp.Kiq.Pool |&gt; Kiq.Pool.checkout() |&gt; Redix.command([&quot;SMEMBERS&quot;, &quot;queues&quot;])","ref":"Kiq.Pool.html","title":"Kiq.Pool","type":"module"},{"doc":"Get the pid of a supervised Redix connection. Connections are randomly allocated and don&#39;t need to be checked back in.","ref":"Kiq.Pool.html#checkout/1","title":"Kiq.Pool.checkout/1","type":"function"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor.","ref":"Kiq.Pool.html#child_spec/1","title":"Kiq.Pool.child_spec/1","type":"function"},{"doc":"","ref":"Kiq.Pool.html#t:options/0","title":"Kiq.Pool.options/0","type":"type"},{"doc":"Job handling is performed by reporters, which are customized GenStage consumers. This module specifies the reporter behaviour and provies an easy way to define new reporters via use Kiq.Reporter. The using macro defines all necessary GenStage functions and defines no-op handlers for all events. Example Custom Reporter Custom reporters may be defined within your application. One common use-case for custom reporters would be reporting exceptions to a tracking service. Here we will define a reporter for Honeybadger: defmodule MyApp.Reporters.Honeybadger do @moduledoc false use Kiq.Reporter @impl Kiq.Reporter def handle_failure(job, error, stacktrace, state) do metadata = Map.take(job, [:jid, :class, :args, :queue, :retry_count]) :ok = Honeybadger.notify(error, metadata, stacktrace) state end end Next you&#39;ll need to specify the reporter inside your application&#39;s Kiq module, using extra_reporters: defmodule MyApp.Kiq do @moduledoc false alias MyApp.Reporters.Honeybadger use Kiq, queues: [default: 50], extra_reporters: [Honeybadger] end This guarantees that your reporter will be supervised by the reporter supervisor, and that it will be re-attached to the reporter producer upon restart if the reporter crashes. Notes About State Handler functions are always invoked with the current state. The return value of each handler function will be used as the updated state after all events are processed.","ref":"Kiq.Reporter.html","title":"Kiq.Reporter","type":"behaviour"},{"doc":"Emitted when job processing has been intentionally aborted.","ref":"Kiq.Reporter.html#c:handle_aborted/3","title":"Kiq.Reporter.handle_aborted/3","type":"callback"},{"doc":"Emitted when an exception ocurred while processing a job.","ref":"Kiq.Reporter.html#c:handle_failure/4","title":"Kiq.Reporter.handle_failure/4","type":"callback"},{"doc":"Emitted when a worker starts processing a job.","ref":"Kiq.Reporter.html#c:handle_started/2","title":"Kiq.Reporter.handle_started/2","type":"callback"},{"doc":"Emitted when the worker has completed the job, regardless of whether it succeded or failed.","ref":"Kiq.Reporter.html#c:handle_stopped/2","title":"Kiq.Reporter.handle_stopped/2","type":"callback"},{"doc":"Emitted when a worker has completed a job and no failures ocurred.","ref":"Kiq.Reporter.html#c:handle_success/3","title":"Kiq.Reporter.handle_success/3","type":"callback"},{"doc":"","ref":"Kiq.Reporter.html#t:options/0","title":"Kiq.Reporter.options/0","type":"type"},{"doc":"","ref":"Kiq.Reporter.html#t:state/0","title":"Kiq.Reporter.state/0","type":"type"},{"doc":"This module simplifies making assertions about enqueued jobs during testing. Testing assertions only work when Kiq is started with test_mode set to :sandbox. In sandbox mode jobs are never flushed to Redis and are stored in memory until the test run is over. Each enqueued job is associated with the process that enqueued it, allowing asynchronous tests to check stored jobs without any interference. Using in Tests If your application has defined a top level Kiq module as MyApp.Kiq, then you would use the testing module inside your application&#39;s case templates like so: use Kiq.Testing, client: MyApp.Kiq.Client That will define two helper functions, assert_enqueued/1 and refute_enqueued/1. The functions can then be used to make assertions on the jobs that have been stored while testing. Given a simple module that enqueues a job: defmodule MyApp.Business do alias MyApp.Kiq def work(args) do Kiq.enqueue(class: &quot;SomeWorker&quot;, args: args) end end The behaviour can be exercised in your test code: defmodule MyApp.BusinessTest do use ExUnit.Case, async: true use Kiq.Testing, client: MyApp.Kiq.Client alias MyApp.Business test &quot;jobs are enqueued with provided arguments&quot; do Business.work([1, 2]) assert_enqueued(class: &quot;SomeWorker&quot;, args: [1, 2]) end end Testing Jobs From Other Processes All calls to assert_enqueued/3 and refute_enqueued/3 use the :sandbox scope by default. That scope ensures that the current process can only find its own enqueued jobs. Sometimes this behavior is undesirable. For example, when jobs are being enqueued outside of the test process. If a separate server or task enqueue a job you may use the :shared scoping to make global assertions. Task.async(fn -&gt; Kiq.enqueue(class: &quot;MyWorker&quot;, args: [1, 2]) end) assert_enqueued(:shared, class: &quot;MyWorker&quot;)","ref":"Kiq.Testing.html","title":"Kiq.Testing","type":"module"},{"doc":"Assert that a job with particular options has been enqueued. Only values for the provided arguments will be checked. For example, an assertion made on class: &quot;MyWorker&quot; will match any jobs for that class, regardless of the args.","ref":"Kiq.Testing.html#assert_enqueued/3","title":"Kiq.Testing.assert_enqueued/3","type":"function"},{"doc":"Refute that a job with particular options has been enqueued. See assert_enqueued/2 for additional details.","ref":"Kiq.Testing.html#refute_enqueued/3","title":"Kiq.Testing.refute_enqueued/3","type":"function"},{"doc":"Defines a behavior and macro to guide the creation of worker modules. Worker modules do the work of processing a job. At a minimum they must define a perform function, which will be called with the arguments that were enqueued with the Kiq.Job. Defining Workers Define a worker to process jobs in the events queue: defmodule MyApp.Workers.Business do use Kiq.Worker, queue: &quot;events&quot;, retry: 10, dead: false @impl Kiq.Worker def perform(args) do IO.inspect(args) end end The perform/1 function will always receive a list of arguments. In this example the worker will simply inspect any arguments that are provided. Enqueuing Jobs All workers implement a new/1 function that converts a list of arguments into a Kiq.Job that is suitable for enqueuing: [&quot;doing&quot;, &quot;business&quot;] |&gt; MyApp.Workers.Business.new() |&gt; MyApp.Kiq.enqueue()","ref":"Kiq.Worker.html","title":"Kiq.Worker","type":"behaviour"},{"doc":"Build a job for this worker using all default options. Any additional arguments that are provided will be merged into the job.","ref":"Kiq.Worker.html#c:new/1","title":"Kiq.Worker.new/1","type":"callback"},{"doc":"The perform/1 function is called with the enqueued arguments. The return value is not important.","ref":"Kiq.Worker.html#c:perform/1","title":"Kiq.Worker.perform/1","type":"callback"},{"doc":"","ref":"Kiq.Worker.html#t:args/0","title":"Kiq.Worker.args/0","type":"type"},{"doc":"","ref":"Kiq.Worker.html#t:opts/0","title":"Kiq.Worker.opts/0","type":"type"},{"doc":"Generates a new reporter. The module will be nested within the lib directory. Custom reporters must be listed as :extra_reporters in your configuration. Examples mix kiq.gen.reporter Kiq.Reporters.Custom","ref":"Mix.Tasks.Kiq.Gen.Reporter.html","title":"Mix.Tasks.Kiq.Gen.Reporter","type":"task"},{"doc":"Generates a new supervisor. The module will be placed within the lib directory. Examples mix kiq.gen.supervisor MyApp.Kiq","ref":"Mix.Tasks.Kiq.Gen.Supervisor.html","title":"Mix.Tasks.Kiq.Gen.Supervisor","type":"task"},{"doc":"Generates a new worker. The module will be nested within the lib directory. Examples mix kiq.gen.worker MyApp.Workers.Business mix kiq.gen.worker MyApp.Workers.Essential -q critical mix kiq.gen.worker MyApp.Workers.Unimportant -r 3 mix kiq.gen.worker MyApp.Workers.Lifeless --no-dead Command Line Options -d, --dead, --no-dead — whether the job will be pushed to the dead set when retries are exhausted -q, --queue — the queue this worker&#39;s jobs will run in -r, --retry — the number of times jobs will be retried","ref":"Mix.Tasks.Kiq.Gen.Worker.html","title":"Mix.Tasks.Kiq.Gen.Worker","type":"task"},{"doc":"Kiq Kiq is a robust and extensible job processing queue that aims for compatibility with Sidekiq, Sidekiq Pro and Sidekiq Enterprise. Job queuing, processing and reporting are all built on GenStage. That means maximum parallelism with the safety of backpressure as jobs are processed. Why Kiq? Many features and architectural choices in Kiq were drawn from existing Elixir job processing packages like Exq, Verk and EctoJob. Each of those packages are great and have varying strenghts, but they lacked seamless interop with Sidekiq Pro or Sidekiq Enterprise jobs. Sidekiq Pro and Enterprise are amazing pieces of commercial software and worth every cent—your organization should buy them! Kiq is intended as a bridge for your team to interop between Ruby and Elixir. As an organization embraces Elixir it becomes necessary to run some background jobs in Elixir, and it must be just as reliable as when jobs were ran through Sidekiq. Sidekiq Pro &amp; Enterprise Comaptible Feature Set Kiq&#39;s feature set includes many marquee offerings from Sidekiq, Sidekiq Pro and Sidekiq Enterprise—plus some additional niceties made possible by running on the BEAM. Here is a table highlighting the Kiq&#39;s features compared to the various Sidekiq versions: FeatureKiqSidekiqSidekiq ProSidekiq Ent Max Size Queues✅❌❌❌ Structured Logging✅❌❌❌ Scheduled Jobs✅✅✅✅ Error Handling✅✅✅✅ Expiring Jobs✅❌✅✅ Worker Metrics✅❌✅✅ Reliable Client✅❌✅✅ Reliable Server✅❌✅✅ Rolling Restarts✅❌❌✅ Periodic Jobs✅❌❌✅ Unique Jobs✅❌❌✅ Leader Election✅❌❌✅ Multi Process✅❌❌✅ Web UI †❌✅✅✅ Batch Jobs ‡❌❌✅✅ Encryption ‡❌❌❌✅ Rate Limiting ‡❌❌❌✅ † Kiq relies on Sidekiq&#39;s Web UI ‡ Planned, but not implemented yet If a feature isn&#39;t supported or planned it is probably for one of these reasons: We get it for free on the BEAM and it isn&#39;t necessary, i.e. (safe shutdown, multi-process, rolling restarts) We enable developers to use custom reporters to do it themselves (stats, error reporting) Design Decisions Avoid global and compile time configuration. All configuration can be defined programatically, eliminating the need for hacks like {:system, &quot;REDIS_URL&quot;}. Not an application, it is included in your application&#39;s supervision tree Testing focused, provide helpers and modes to aid testing Extensible job handling via GenStage consumers Simplified worker definitions to ease job definition and pipelining","ref":"readme.html","title":"Kiq","type":"extras"},{"doc":"Add kiq to your list of dependencies in mix.exs: def deps do [ {:kiq, &quot;~&gt; 0.7&quot;} ] end Then run mix deps.get to install the dependency. Finally, add the supervisor to your application&#39;s supervision tree: {MyApp.Kiq, []} Kiq itself is not an application and must be started within your application&#39;s supervision tree. All of your application&#39;s configuration and custom methods should be put into the supervisor.","ref":"readme.html#installation","title":"Kiq - Installation","type":"extras"},{"doc":"Kiq isn&#39;t an application that must be started. Similarly to Ecto, you define one or more Kiq modules within your application. This allows multiple supervision trees with entirely different configurations. Run the generator to define a Kiq supervisor for your application: mix kiq.gen.supervisor MyApp.Kiq Include the module in your application&#39;s supervision tree: defmodule MyApp.Application do @moduledoc false use Application alias MyApp.{Endpoint, Kiq, Repo} def start(_type, _args) do children = [ {Repo, []}, {Endpoint, []}, {Kiq, []} ] Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor) end end With the supervision tree in place you are ready to start creating workers! The simplest way to create a worker is through the generator: mix kiq.gen.worker MyApp.Workers.Business That will define a worker with a perform/1 function where all the magic will happen. See mix help kiq.gen.worker for additional options. Check the hexdocs for additional details, configuration options, how to test, defining workers and custom reporters.","ref":"readme.html#usage","title":"Kiq - Usage","type":"extras"},{"doc":"Kiq has a set of benchmarks to track the performance of important operations. Benchmarks are ran using the Benchee library and require Redis to be running. To run all benchmarks: mix run bench/bench_helper.exs","ref":"readme.html#benchmarks","title":"Kiq - Benchmarks","type":"extras"},{"doc":"Clone the repository and run $ mix test to make sure everything is working. For tests to pass, you must have a Redis server running on localhost, port 6379, database 3. You can configure a different host, port and database by setting the REDIS_URL environment variable before testing. Note that tests will wipe the the configured database on the Redis server multiple times while testing. By default database 3 is used for testing.","ref":"readme.html#contributing","title":"Kiq - Contributing","type":"extras"},{"doc":"Kiq is released under the MIT license. See the LICENSE.","ref":"readme.html#license","title":"Kiq - License","type":"extras"},{"doc":"Changelog All notable changes to this project will be documented in this file. The format is based on Keep a Changelog and this project adheres to Semantic Versioning.","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Update and loosen dependency constraints, particularly for nimble parsec","ref":"changelog.html#0-7-1-2020-05-06","title":"Changelog - [0.7.1] — 2020-05-06","type":"extras"},{"doc":"Added [Reporter.Instrumenter] Bump telemetry version.","ref":"changelog.html#0-7-0-2019-05-13","title":"Changelog - [0.7.0] - 2019-05-13","type":"extras"},{"doc":"Added [Kiq.Encoder] Wrap Jason.encode and attempt to sanitize terms that can&#39;t be represented as JSON. This is primarily a safeguard against enqueuing jobs with structs or tuples as arguments. Fixed [Kiq.Client.Resurrection] Correct usage of SCAN to ensure all backup queues will be found for re-enqueueing. Changed [Kiq.Job] Keys within job arguments are no longer converted to atoms. This is for safe execution and is in line with how Phoenix passes controller arguments. [Kiq.Client] Enqueue all jobs in a single pipelined command. This moves all of the unique, scheduling and enqueueing logic into a lua script. The script reduces the number of round trips to redis and enabled pipelining. Overall this improved job enqueue/execute performance from 1800 jobs/sec to slightly more than 3700 jobs/sec.","ref":"changelog.html#0-6-0-2019-03-18","title":"Changelog - [0.6.0] - 2019-03-18","type":"extras"},{"doc":"Added [Kiq.Periodic] Periodic jobs support! Periodic (AKA &quot;Cron&quot;) jobs are registered with a schedule and enqueued automatically by Kiq accordingly. This is a fully fledged alternative to using a separate scheduler such as Quantum. Changed [Kiq.Reporter.Logger] Job lifecycle logging uses the format event: &quot;job_status&quot; instead of status: &quot;status&quot;. For example, if you were checking logs for &quot;status&quot;:&quot;finished&quot; you would now look for &quot;event&quot;:&quot;job_finished&quot;. Fixed [Kiq.Client] Eliminate use of MUTLI/EXEC for stats and queuing. This caused a race condition that could prevent stats being recorded and jobs from being enqueued.","ref":"changelog.html#0-5-0-2018-12-14","title":"Changelog - 0.5.0 — 2018-12-14","type":"extras"},{"doc":"Added [Reporter.Retryer] Dead jobs support! Jobs with exhausted retries are moved into the dead set. Workers or jobs may be configured with dead: false to prevent being moved to the dead set. [Kiq] Quiet support! The configure/1 function can be used to set runtime configuration. Calling MyKiq.configure(quiet: true) will stop all queues from starting new jobs while allowing currently running jobs to finish. This can be used for graceful shutdown and smooth blue/green deployment. [Reporter.Instrumenter] A new reporter that provides instrumentation data through Telemetry. Reporting job execution metrics only requires attaching to [:kiq, :job, event] events. [mix kiq.gen.supervisor] A new generator to create an initial Kiq supervisor module. [mix kiq.gen.reporter] A new generator to create custom reporters with all of the callbacks defined. [mix kiq.gen.worker] A new generator to create workers with retries, queue and dead specified. Changed [Script] More functionality, including dequeueing and descheduling, has been moved into Lua scripts. To reduce bandwidth and io overhead all EVAL usage has been replaced with EVALSHA. [Identity] For purely aesthetic reasons the nonce value is now lower case. [Logger] Consistently format logs as JSON. Previously some ad-hoc logs were unstructured, which made parsing difficult. Additionally, logs now contain &quot;source&quot;:&quot;kiq&quot; for easier identification. Fixed [Kiq] In progress backup queues, AKA &quot;Private Queues&quot;, are now implemented with a hash rather than a list. This ensures that a job can always be removed from the backup, regardless of how it is encoded. Previously, when a job was enqueued by Sidekiq the serialized version wouldn&#39;t match the version serialized by Kiq. The LREM command requires an exact binary match or list elements won&#39;t be removed, which caused jobs to linger in the backup queue.","ref":"changelog.html#0-4-0-2018-12-03","title":"Changelog - 0.4.0 — 2018-12-03","type":"extras"},{"doc":"Added [Kiq] Expiring Job support. Expiring jobs won&#39;t be ran after a configurable amount of time. The expiration period is set with the expires_in option, which accepts millisecond values identically to the unique_for option. [Kiq.Pool] Connection pooling. Recently added benchmarks quickly identified a bottleneck in the single Client/Redis connection. A simple random-order pool was introduced that allowed the client, queues and reporters to rotate through a set of connections. This change alone doubled the enqueue/dequeue throughput. [Kiq.Client] Reliable push support. Enqueued jobs are now buffered in memory and periodically flushed to Redis. If there are any connection errors or Redis is down the jobs are retained and flushing is retried with backoff. [Kiq.Testing] Sandbox test mode. When test_mode is set to :sandbox jobs will never be flushed to Redis. Each locally buffered job is associated with the process that enqueued it, enabling concurrent testing with isolation between test runs. [Kiq] Leadership election. Useful when coordinating work that should only happen on a single node. Internally used to prevent duplicate job resurrection, and in the future will be used for periodic jobs. [Kiq] Private queues. Ensure terminated jobs are resurrected when the application starts up. Unlike the previous job backup mechanism this guarantees that only jobs from dead processors are resurrected; in-process jobs will never be duplicated. Changed [Redix] Upgrade from Redix 0.7.X to 0.8.X, which introduced the noreply_ variant of commands. [Kiq.Config] Renamed poll_interval option to flush_interval, and the default changed from 1000 to 500. Fixed [Kiq] Failing unique jobs would never be retried.","ref":"changelog.html#0-3-0-2018-10-28","title":"Changelog - 0.3.0 — 2018-10-28","type":"extras"},{"doc":"Added [Kiq] Unique Job support! Workers, and their corresponding jobs, can now be made unique within for a period of time. Until the initial job has succeeded it is considered &quot;locked&quot; and all subsequent attempts to enqueue a job with the same class, queue and args will be ignored. The unique period is specified with the unique_for option, which accepts millisecond values for easy interaction with the :timer module&#39;s minutes/1 and seconds/1 functions. Fixed [Kiq.Runner] Correctly convert nested modules binaries into the correct worker, i.e. convert &quot;Kiq.Workers.Business&quot; into the module Kiq.Workers.Business. [Kiq.Job] Correctly serialize pid and job payloads when recording a worker&#39;s running jobs for heartbeats. Incorrectly formatted hashes in Redis will cause the Sidekiq &quot;busy&quot; dashboard to crash. [Kiq.Heartbeat] Retain the original ran_at value for reported jobs. Running jobs are now encoded when they are started, preventing repeated JSON encodings and allowing the initial ran_at value to be used when reporting. Prior to this change the job was always reported as being started &quot;Just Now&quot;. [Kiq.Job] Skip serializing retry_count when the value is 0. Sidekiq doesn&#39;t include a retry_count when there hasn&#39;t been a retry, which prevents the job serialized by Kiq from matching up. If the serialized job doesn&#39;t match up then the job can&#39;t be removed from the backup queue, leading to a buildup of duplicate jobs.","ref":"changelog.html#0-2-0-2018-09-17","title":"Changelog - 0.2.0 — 2018-09-17","type":"extras"},{"doc":"Added Initial release, everything was added! Job creation Worker definition Running and configuring queues Retries Scheduled jobs Statistics Custom reporters Testing support Heartbeat reporting","ref":"changelog.html#0-1-0-2018-07-31","title":"Changelog - 0.1.0 - 2018-07-31","type":"extras"}]